==> oobium_datastore_gen-1.0.0.tar.gz: lib/src/generators/library.dart <==
import 'package:collection/collection.dart';
import 'package:oobium_datastore_gen/src/feature.dart';
import 'package:oobium_datastore_gen/src/generators/class.dart';
import 'package:oobium_datastore_gen/src/generators/converter.dart';
import 'package:oobium_datastore_gen/src/generators/models_feature_collection.dart';
import 'package:oobium_datastore_gen/src/generators/models_feature_model.dart';
import 'package:oobium_datastore_gen/src/generators/persist_feature.dart';
import 'package:oobium_datastore_gen/src/generators/rest_feature.dart';
import 'package:oobium_datastore_gen/src/generators/trigger_feature.dart';
import 'package:oobium_datastore_gen/src/generators/models_feature.dart';
import 'package:oobium_datastore_gen/src/schema_parser.dart';

List<String> genImports(Schema schema) {
  final imports = <String, Set<String>>{};
  schema.imports.forEach((import) {
    final importPackage = import.from;
    if(importPackage != null) {
      imports.putIfAbsent(importPackage, () => {}).addAll(import.names);
    }
  });
  return imports.entries.map((e) => "import '${e.key}' show ${e.value.sorted((a,b) => a.compareTo(b)).join(', ')};").toList();
}

List<String> genFeatureParts(Schema schema) => schema.features.where((f) => f.isPart).map((f) {
  return "part '${[
    schema.name,
    if(schema.isVersioned) schema.version,
    'schema.g.feature_${f.name}.dart'
  ].join('.')}';";
}).toList();

String genXPart(Schema schema) => "part '${[
    schema.name,
    if(schema.isVersioned) schema.version,
    'schema.x.dart'
  ].join('.')}';";

String genLibrary(Schema schema) {

  final dsName = schema.dsName;
  final classes = schema.classes;
  final models = schema.models;
  final objects = schema.objects;
  final types = schema.types;

  final imports = <String>[
    ...genImports(schema),
    "import 'dart:async';",
    "import 'package:oobium_datastore/oobium_datastore.dart';",
    "import 'package:oobium_logging/oobium_logging.dart';",
  ].toSet().toList()..sort();

  final parts = <String>[
    ...genFeatureParts(schema),
    genXPart(schema)
  ]..sort();

  final params = <String>[
    if(schema.has<PersistFeature>())
      'String? path, OnUpgrade? onUpgrade',
    if(schema.has<RestFeature>())
      ...models.map((m) => '${m.adapterName}? ${m.adapterVarName}'),
  ].join(',');

  final features = (<String>[
    '_ModelFeature()',
    if(schema.has<TriggerFeature>())
      '_TriggerFeature()',
    if(schema.has<PersistFeature>())
      'if(path != null) ${PersistFeatureGenerator(schema).compileCreator()}',
    if(schema.has<RestFeature>())
      '${RestFeatureGenerator(schema).compileCreator()}',
  ]..sort()).join(',');

  final superArgs = [
    'StateContext(features: [$features])',
    'const _Converter()',
  ].join(',');

  final initializers =  [
    if(schema.has<RestFeature>())
      'feature<_RestFeature>()._modelContext = this;'
  ];

  final initializer = initializers.isEmpty ? ';' : '{${initializers.join()}}';

  return
'''
${imports.join('\n')}
${parts.join('\n')}
final _log = Logger($dsName);

class $dsName extends ModelContext {
  $dsName(${params.isEmpty ? '' : '{$params}'})
    : super($superArgs)$initializer
  
  Future<$dsName> open() => super.open().then((_) => this);
  
  ${objects.map((m) => '${m.refName} get ${m.varName} => feature<_ModelFeature>().${m.varName};').join()}
  ${types.map((m) => '${m.namePlural} get ${m.varNamePlural} => feature<_ModelFeature>().${m.varNamePlural};').join()}
  
  void load(data) => feature<_ModelFeature>().load(data);
  void unload(data) => feature<_ModelFeature>().unload(data);
}
${ConverterGenerator(schema).compile()}
${classes.map((c) => ClassGenerator(c).compile()).join('\n')}
${models.map((m) => ModelGenerator(m).compile()).join('\n')}
${types.map((m) => ModelCollectionGenerator(m).compile()).join('\n')}
${ModelFeatureGenerator(schema).compile()}
${TriggerFeatureGenerator(schema).compile()}
''';
}

==> oobium_datastore_gen-1.1.1.tar.gz: lib/src/generators/library.dart <==
import 'package:collection/collection.dart';
import 'package:oobium_datastore_gen/src/feature.dart';
import 'package:oobium_datastore_gen/src/generators/class.dart';
import 'package:oobium_datastore_gen/src/generators/converter.dart';
import 'package:oobium_datastore_gen/src/generators/models_feature_collection.dart';
import 'package:oobium_datastore_gen/src/generators/models_feature_model.dart';
import 'package:oobium_datastore_gen/src/generators/persist_feature.dart';
import 'package:oobium_datastore_gen/src/generators/rest_feature.dart';
import 'package:oobium_datastore_gen/src/generators/trigger_feature.dart';
import 'package:oobium_datastore_gen/src/generators/models_feature.dart';
import 'package:oobium_datastore_gen/src/schema_parser.dart';

List<String> genImports(Schema schema) {
  final imports = <String, Set<String>>{};
  schema.imports.forEach((import) {
    final importPackage = import.from;
    if(importPackage != null) {
      imports.putIfAbsent(importPackage, () => {}).addAll(import.names);
    }
  });
  return imports.entries.map((e) => "import '${e.key}' show ${e.value.sorted((a,b) => a.compareTo(b)).join(', ')};").toList();
}

List<String> genFeatureParts(Schema schema) => schema.features.where((f) => f.isPart).map((f) {
  return "part '${[
    schema.name,
    if(schema.isVersioned) schema.version,
    'schema.g.feature_${f.name}.dart'
  ].join('.')}';";
}).toList();

String genXPart(Schema schema) => "part '${[
    schema.name,
    if(schema.isVersioned) schema.version,
    'schema.x.dart'
  ].join('.')}';";

String genLibrary(Schema schema) {

  final dsName = schema.dsName;
  final classes = schema.classes;
  final models = schema.models;
  final objects = schema.objects;
  final types = schema.types;

  final imports = <String>[
    ...genImports(schema),
    "import 'dart:async';",
    "import 'package:oobium_datastore/oobium_datastore.dart';",
    "import 'package:oobium_logging/oobium_logging.dart';",
  ].toSet().toList()..sort();

  final parts = <String>[
    ...genFeatureParts(schema),
    genXPart(schema)
  ]..sort();

  final params = <String>[
    if(schema.has<PersistFeature>())
      'String? path, OnUpgrade? onUpgrade',
    if(schema.has<RestFeature>())
      ...models.map((m) => '${m.restAdapterName}? ${m.restAdapterVarName}'),
    if(schema.has<TriggerFeature>())
      ...models.map((m) => '${m.triggersName}? ${m.triggersVarName}'),
  ].join(',');

  final features = (<String>[
    '_ModelFeature()',
    if(schema.has<TriggerFeature>())
      TriggerFeatureGenerator(schema).compileCreator(),
    if(schema.has<PersistFeature>())
      'if(path != null) ${PersistFeatureGenerator(schema).compileCreator()}',
    if(schema.has<RestFeature>())
      RestFeatureGenerator(schema).compileCreator(),
  ]..sort()).join(',');

  final superArgs = [
    'StateContext(features: [$features])',
    'const _Converter()',
  ].join(',');

  final initializers =  [
    if(schema.has<RestFeature>())
      'feature<_RestFeature>()._modelContext = this;',
    if(schema.has<TriggerFeature>())
      'feature<_TriggerFeature>()._modelContext = this;',
  ];

  final initializer = initializers.isEmpty ? ';' : '{${initializers.join()}}';

  return
'''
${imports.join('\n')}
${parts.join('\n')}
final _log = Logger($dsName);

class $dsName extends ModelContext {
  $dsName(${params.isEmpty ? '' : '{$params}'})
    : super($superArgs)$initializer
  
  Future<$dsName> open() => super.open().then((_) => this);
  
  ${objects.map((m) => '${m.refName} get ${m.varName} => feature<_ModelFeature>().${m.varName};').join()}
  ${types.map((m) => '${m.namePlural} get ${m.varNamePlural} => feature<_ModelFeature>().${m.varNamePlural};').join()}
  
  void load(data) => feature<_ModelFeature>().load(data);
  void unload(data) => feature<_ModelFeature>().unload(data);
}
${ConverterGenerator(schema).compile()}
${classes.map((c) => ClassGenerator(c).compile()).join('\n')}
${models.map((m) => ModelGenerator(m).compile()).join('\n')}
${types.map((m) => ModelCollectionGenerator(m).compile()).join('\n')}
${ModelFeatureGenerator(schema).compile()}
''';
}
